"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.groupJoin = void 0;
const BasicEnumerable_1 = require("../BasicEnumerable");
const groupJoin = (outer, inner, outerKeySelector, innerKeySelector, resultSelector, comparer) => {
    const generator = comparer ?
        groupJoinWithComparer(outer, inner, outerKeySelector, innerKeySelector, resultSelector, comparer) :
        groupJoinSimple(outer, inner, outerKeySelector, innerKeySelector, resultSelector);
    return new BasicEnumerable_1.BasicEnumerable(generator);
};
exports.groupJoin = groupJoin;
const groupJoinWithComparer = (outer, inner, outerKeySelector, innerKeySelector, resultSelector, comparer) => {
    return function* () {
        var _a;
        const innerKeyMap = [];
        for (const innerValue of inner) {
            const key = innerKeySelector(innerValue);
            const record = innerKeyMap.find(x => comparer(x.key, key));
            if (record) {
                record.values.push(innerValue);
            }
            else {
                innerKeyMap.push({ key, values: [innerValue] });
            }
        }
        for (const outerValue of outer) {
            const key = outerKeySelector(outerValue);
            const innerRecord = (_a = innerKeyMap.find(x => comparer(x.key, key))) !== null && _a !== void 0 ? _a : { key, values: [] };
            yield resultSelector(outerValue, innerRecord.values);
        }
    };
};
const groupJoinSimple = (outer, inner, outerKeySelector, innerKeySelector, resultSelector) => {
    return function* () {
        var _a;
        const innerKeyMap = new Map();
        for (const innerValue of inner) {
            const key = innerKeySelector(innerValue);
            const values = innerKeyMap.get(key);
            if (values) {
                values.push(innerValue);
            }
            else {
                innerKeyMap.set(key, [innerValue]);
            }
        }
        for (const outerValue of outer) {
            const key = outerKeySelector(outerValue);
            const values = (_a = innerKeyMap.get(key)) !== null && _a !== void 0 ? _a : [];
            yield resultSelector(outerValue, values);
        }
    };
};
